@@ -374,6 +374,8 @@ class Pipeline(object):  # pylint: disable=too-many-instance-attributes
             except (JobError, InfrastructureError) as exc:
                 action.errors = exc.message
                 # set results including retries
+                if "boot-result" not in action.data:
+                    action.data['boot-result'] = 'failed'
                 action.results = {"fail": exc}
                 self._diagnose(connection)
                 action.cleanup()
@@ -649,15 +651,22 @@ class Action(object):  # pylint: disable=too-many-instance-attributes
         if 'protocols' not in self.parameters:
             return
         for protocol in self.job.protocols:
-            for params in self.parameters['protocols'][protocol.name]:
-                for call in [
-                        params for name in params
-                        if name == 'action' and params[name] == self.name]:
-                    reply = protocol(call)
-                    message = protocol.collate(reply, params)
-                    self.logger.debug(
-                        "Setting common data key %s to %s"
-                        % (message[0], message[1]))
+            if protocol.name not in self.parameters['protocols']:
+                # nothing to do for this action with this protocol
+                continue
+            params = self.parameters['protocols'][protocol.name]
+            for call_dict in [call for call in params if 'action' in call and call['action'] == self.name]:
+                del call_dict['yaml_line']
+                if 'message' in call_dict:
+                    del call_dict['message']['yaml_line']
+                if 'timeout' in call_dict:
+                    del call_dict['timeout']['yaml_line']
+                protocol.check_timeout(self.connection_timeout.duration, call_dict)
+                self.logger.info("Making protocol call for %s using %s", self.name, protocol.name)
+                reply = protocol(call_dict)
+                message = protocol.collate(reply, call_dict)
+                if message:
+                    self.logger.info("Setting common data key %s to %s", message[0], message[1])
                     self.set_common_data(protocol.name, message[0], message[1])
 
     def run(self, connection, args=None):
@@ -766,7 +775,7 @@ class Action(object):  # pylint: disable=too-many-instance-attributes
         if not connection.connected:
             self.logger.debug("Already disconnected")
             return
-        self.logger.debug("%s: Wait for prompt. %s seconds" % (self.name, int(self.timeout.duration)))
+        self.logger.debug("%s: Wait for prompt. %s seconds" % (self.name, int(self.connection_timeout.duration)))
         connection.wait()
 
 
